@@ -12,9 +12,11 @@ from pre_commit.util import CalledProcessError
 from pre_commit.util import cmd_output
 from pre_commit.util import cmd_output_b
 
-
 logger = logging.getLogger(__name__)
 
+# see #2046
+NO_FS_MONITOR = ('-c', 'core.useBuiltinFSMonitor=false')
+
 
 def zsplit(s: str) -> List[str]:
     s = s.strip('\0')
@@ -39,9 +41,10 @@ def no_git_env(
     return {
         k: v for k, v in _env.items()
         if not k.startswith('GIT_') or
+        k.startswith(('GIT_CONFIG_KEY_', 'GIT_CONFIG_VALUE_')) or
         k in {
             'GIT_EXEC_PATH', 'GIT_SSH', 'GIT_SSH_COMMAND', 'GIT_SSL_CAINFO',
-            'GIT_SSL_NO_VERIFY',
+            'GIT_SSL_NO_VERIFY', 'GIT_CONFIG_COUNT',
         }
     }
 
@@ -52,10 +55,10 @@ def get_root() -> str:
     # "rev-parse --show-cdup" to get the appropriate path, but must perform
     # an extra check to see if we are in the .git directory.
     try:
-        root = os.path.realpath(
+        root = os.path.abspath(
             cmd_output('git', 'rev-parse', '--show-cdup')[1].strip(),
         )
-        git_dir = os.path.realpath(get_git_dir())
+        git_dir = os.path.abspath(get_git_dir())
     except CalledProcessError:
         raise FatalError(
             'git failed. Is it installed, and are you in a Git repository '
@@ -155,12 +158,15 @@ def get_all_files() -> List[str]:
 
 
 def get_changed_files(old: str, new: str) -> List[str]:
-    return zsplit(
-        cmd_output(
-            'git', 'diff', '--name-only', '--no-ext-diff', '-z',
-            f'{old}...{new}',
-        )[1],
-    )
+    diff_cmd = ('git', 'diff', '--name-only', '--no-ext-diff', '-z')
+    try:
+        _, out, _ = cmd_output(*diff_cmd, f'{old}...{new}')
+    except CalledProcessError:  # pragma: no cover (new git)
+        # on newer git where old and new do not have a merge base git fails
+        # so we try a full diff (this is what old git did for us!)
+        _, out, _ = cmd_output(*diff_cmd, f'{old}..{new}')
+
+    return zsplit(out)
 
 
 def head_rev(remote: str) -> str:
@@ -182,10 +188,11 @@ def init_repo(path: str, remote: str) -> None:
     if os.path.isdir(remote):
         remote = os.path.abspath(remote)
 
+    git = ('git', *NO_FS_MONITOR)
     env = no_git_env()
     # avoid the user's template so that hooks do not recurse
-    cmd_output_b('git', 'init', '--template=', path, env=env)
-    cmd_output_b('git', 'remote', 'add', 'origin', remote, cwd=path, env=env)
+    cmd_output_b(*git, 'init', '--template=', path, env=env)
+    cmd_output_b(*git, 'remote', 'add', 'origin', remote, cwd=path, env=env)
 
 
 def commit(repo: str = '.') -> None:
